﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Services.Optimization.Billing.Model {
  public class ContactInformation {
    public long ContactInformationId { get; set; }
    public string OrganizationName { get; set; }
    public string Street { get; set; }
    public string City { get; set; }
    public string State { get; set; }
    public string PostalCode { get; set; }
    public string LastName { get; set; }
    public string FirstName { get; set; }
    public string Email { get; set; }
  }

  public class PaymentInformation {
    public long PaymentInformationId { get; set; }
    public string CardNumber { get; set; }
    public PaymentMethod PaymentMethod { get; set; }
    public string AdditionalInformation { get; set; }
  }

  public enum PaymentMethod {
    Cheque,
    Visa,
    MasterCard,
    PayPal,
    Diners
  }

  public enum BillingType {
    Pre,
    Post
  }

  public enum BillingPeriod {
    Weekly,
    Monthly,
    Yearly
  }

  public class Product {
    public long ProductId { get; set; }
    public string Name { get; set; }
    public string Description { get; set; }
    public string ProductType { get; set; }
    public double Price { get; set; }
  }

  public class User {
    public long UserId { get; set; }
    public string Name { get; set; }
    public virtual IList<PaymentInformation> PaymentInformation { get; set; }
  }

  public enum OrderState {
    Created,
    Active,
    Suspended,
    Overdue,
    Finished
  }

  public class Order {
    public long OrderId { get; set; }
    public long UserId { get; set; }
    public BillingType BillingType { get; set; }
    public long OrderStateId { get; set; }
    public BillingPeriod BillingPeriod { get; set; }
    public DateTime? ActiveSince { get; set; }
    public DateTime? ActiveUntil { get; set; }


    public virtual User User { get; set; }
    public virtual OrderState State { get; set; }
    public virtual IList<Invoice> Invoices { get; set; }
    public virtual IList<OrderLine> OrderLines { get; set; }
  }

  public class OrderLine {
    public long OrderLineId { get; set; }
    public long OrderId { get; set; }
    public long ProductId { get; set; }
    public int Quantity { get; set; }
    public double ProductPrice { get; set; }

    public virtual Order Order { get; set; }
    public virtual Product Product { get; set; }
  }

  public class Invoice {
    public long InvoiceId { get; set; }
    public long UserId { get; set; }
    public long OrderId { get; set; }
    public DateTime Due { get; set; }
    public string Status { get; set; }
    public string InvoiceDocument { get; set; }

    public virtual User User { get; set; }
    public virtual Order Order { get; set; }
    public virtual IList<InvoiceLine> InvoiceLines { get; set; }
  }

  public class InvoiceLine {
    public long InvoiceLineId { get; set; }
    public long InvoiceId { get; set; }
    public long ProductId { get; set; }
    public int Quantity { get; set; }
    public double ProductPrice { get; set; }

    public virtual Invoice Invoice { get; set; }
    public virtual Product Product { get; set; }
  }

  public class UsageRecord {
    public long UsageRecordId { get; set; }
    public long UserId { get; set; }
    public long ProductId { get; set; }
    public DateTime Begin { get; set; }
    public DateTime End { get; set; }
    public int ServiceIdentifier { get; set; }
    public int ResourceIdentifier { get; set; }

    public virtual User User { get; set; }
    public virtual Product Product { get; set; }
  }
}
